/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package txrxcan_noirq;

//import JNA Library
import com.sun.jna.Library;
import com.sun.jna.Native;
import com.sun.jna.Platform;

/**
 *
 * @author 
 */
public class TxRxCAN_NoIRQ {
    //windows dll for printf function
    public interface CLibrary extends Library {
        CLibrary INSTANCE = (CLibrary)Native.loadLibrary((Platform.isWindows() ? "msvcrt" : "c"),CLibrary.class);

        void printf(String format, Object... args);
    }
    
    //PISO-CAN DLL function
    public interface PISOCANLibrary extends Library {
        PISOCANLibrary INSTANCE = (PISOCANLibrary)Native.loadLibrary((Platform.isWindows() ? "pisocan" : "c"),PISOCANLibrary.class);

        short CAN_GetDllVersion();
        short CAN_TotalBoard();
        short CAN_GetCardPortNum(byte BoardNo, byte[] bGetPortNum);
        short CAN_ActiveBoard(byte BoardNo);
        short CAN_CloseBoard(byte BoardNo);
        short CAN_BoardIsActive(byte BoardNo);
        short CAN_Reset(byte BoardNo,byte Port);
        short CAN_Init(byte BoardNo,byte Port);
        short CAN_ConfigWithoutStruct(byte BoardNo, byte Port, int AccCode, int AccMask, byte BaudRate, byte BT0, byte BT1);
        short CAN_EnableRxIrq(byte BoardNo,byte Port);
        short CAN_DisableRxIrq(byte BoardNo,byte Port);
        short CAN_RxIrqStatus(byte BoardNo,byte Port,byte[] bStatus);
        short CAN_InstallIrq(byte BoardNo);
        short CAN_RemoveIrq(byte BoardNo);
        short CAN_IrqStatus(byte BoardNo,byte[] bStatus);
        short CAN_Status(byte BoardNo, byte Port,byte[] bStatus);
		short CAN_BusErrorCode(byte BoardNo, byte Port,byte[] bErrorCode);
        
        short CAN_SendWithoutStruct(byte BoardNo, byte Port, byte Mode, int Id, byte Rtr, byte Dlen, byte[] Data);
        
        short CAN_RxMsgCount(byte BoardNo, byte Port);
        short CAN_ReceiveWithoutStruct(byte BoardNo, byte Port, byte[] Mode, int[] Id, byte[] Rtr, byte[] Dlen, byte[] Data, int[] H_MsgTimeStamps ,int[] L_MsgTimeStamps);
        
        short CAN_ClearSoftBuffer(byte BoardNo, byte Port);
        short CAN_ClearDataOverrun(byte BoardNo, byte Port);

        void  CAN_OutputByte(byte BoardNo,byte Port,int wOffset,byte bValue);
        byte  CAN_InputByte(byte BoardNo,byte Port,int wOffset);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
        short ver = 0, sret = 0, i = 0;
        byte B0 = 0;        //Board 0
        byte P1 = 1, P2 = 2;  //Port 1, 2
        int AccCode = 0, AccMask = 0;
        byte BaudRate = 0, BT0 = 0, BT1 = 0;
        byte CAN_TxMode = 0, CAN_TxRtr = 0, CAN_TxDlen = 0;
        byte [] CAN_RxMode = new byte[1];
        byte [] CAN_RxRtr = new byte[1];
        byte [] CAN_RxDlen = new byte[1];
        byte [] CAN_TxData = new byte[8];
        byte [] CAN_RxData = new byte[8];
        int CAN_TxId = 0;
        int [] CAN_RxId = new int[1];;
        int [] CAN_RxTimeStamp_H = new int[1];;
        int [] CAN_RxTimeStamp_L = new int[1];;
        int LoopCount = 0, TimeoutCount=0;
        boolean Flag_TxCANData = false, Flag_GetRxData = false;
        
        //Get PISO-CAN Dll version
        ver = PISOCANLibrary.INSTANCE.CAN_GetDllVersion();
        CLibrary.INSTANCE.printf("PISO-CAN Demo, Library version: v%X\n", ver);
        
        //Active Board
        sret = PISOCANLibrary.INSTANCE.CAN_ActiveBoard(B0);
        if(sret != 0){
            CLibrary.INSTANCE.printf("CAN_ActiveBoard error, ret=%d\n", sret);
            return;
        }
        
        //Reset Port 1
        sret = PISOCANLibrary.INSTANCE.CAN_Reset(B0, P1);
        if(sret != 0){
            CLibrary.INSTANCE.printf("CAN_Reset P%d error, ret=%d\n", P1, sret);
            return;
        }        
        
        //Configure Port 1
        AccCode = 0x0;
        AccMask = 0xffffffff;
        BaudRate = 4; //0:user define 1=10K   2=20K   3=50K  4=125K  5=250K  6=500K 7=800K  8=1Mbps
        BT0 = 0;
        BT1 = 0;
        
        sret = PISOCANLibrary.INSTANCE.CAN_ConfigWithoutStruct(B0, P1, AccCode, AccMask, BaudRate, BT0, BT1);
        if(sret != 0){
            CLibrary.INSTANCE.printf("CAN_ConfigWithoutStruct P%d error, ret=%d\n", P1, sret);
            return;
        } 
        
        //Reset Port 2
        sret = PISOCANLibrary.INSTANCE.CAN_Reset(B0, P2);
        if(sret != 0){
            CLibrary.INSTANCE.printf("CAN_Reset P%d error, ret=%d/r/n", P2, sret);
            return;
        }
        
        //Configure Port 2
        AccCode = 0x0;
        AccMask = 0xffffffff;
        BaudRate = 4; //0:user define 1=10K   2=20K   3=50K  4=125K  5=250K  6=500K 7=800K  8=1Mbps
        BT0 = 0;
        BT1 = 0;
        
        sret = PISOCANLibrary.INSTANCE.CAN_ConfigWithoutStruct(B0, P2, AccCode, AccMask, BaudRate, BT0, BT1);
        if(sret != 0){
            CLibrary.INSTANCE.printf("CAN_ConfigWithoutStruct P%d error, ret=%d\n", P2, sret);
            return;
        } 
        
        //Use P1 to Send CAN Messages
        //Use P2 to Receive CAN Messages
        CAN_TxMode = 0;   // CAN 2.0A
        CAN_TxRtr = 0;    //No remote frame
        CAN_TxDlen = 8;   
        CAN_TxId = 0;     //CAN Id
        for(i = 0; i < 8 ; i ++){
            CAN_TxData[i] = 0;
        }
        
        Flag_TxCANData = true;
        LoopCount = 0;
        TimeoutCount = 200; //ms
        
        while((Flag_TxCANData == true) && (LoopCount < 100)){
            //Send CAN Message to Port 1
            sret = PISOCANLibrary.INSTANCE.CAN_SendWithoutStruct(B0, P1, CAN_TxMode, CAN_TxId, CAN_TxRtr, CAN_TxDlen, CAN_TxData);
            if(sret != 0){
                CLibrary.INSTANCE.printf("CAN_SendWithoutStruct P%d error, ret=%d\n", P1, sret);
                Flag_TxCANData = false;
            } 
            else{    
                CLibrary.INSTANCE.printf("P%d CAN Send Message\n", P1);
            
                //Receive CAN Message from Port 2
                Flag_GetRxData = false;
                TimeoutCount = 200;
                while((TimeoutCount > 0) && (Flag_GetRxData == false)){
                    sret = PISOCANLibrary.INSTANCE.CAN_RxMsgCount(B0, P2);
                    if(sret > 0){
                        sret = PISOCANLibrary.INSTANCE.CAN_ReceiveWithoutStruct(B0, P2, CAN_RxMode, CAN_RxId, CAN_RxRtr, CAN_RxDlen, CAN_RxData, CAN_RxTimeStamp_H, CAN_RxTimeStamp_L);
                        if(sret > 0){
                            CLibrary.INSTANCE.printf("CAN_ReceiveWithoutStruct P%d error, ret=%d\n", P2, sret);
                            Flag_TxCANData = false;
                        }
                        else{    
                            CLibrary.INSTANCE.printf("[P%d CAN_Receice]: Mode:%01X, ID:%08X, Rtr:%01X, DL:%01X ", P2, CAN_RxMode[0], CAN_RxId[0], CAN_RxRtr[0], CAN_RxDlen[0]);
                            if(CAN_RxDlen[0] > 0){
                                CLibrary.INSTANCE.printf("Data: ");
                                for(i = 0 ; i < CAN_RxDlen[0] ; i++){
                                    CLibrary.INSTANCE.printf("%02X ", CAN_RxData[i]);
                                }
                                CLibrary.INSTANCE.printf("\n");
                            }
                            Flag_GetRxData = true;
                        }
                    }

                    try {
                        Thread.sleep(1);    //sleep 1ms
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }

                    CAN_TxId++;     //CAN Id
                    for(i = 0; i < 8 ; i ++){
                        CAN_TxData[i]++;
                    }
                    LoopCount++;
                }
            }
        }
        
        
        //Close Board
        sret = PISOCANLibrary.INSTANCE.CAN_CloseBoard(B0);
        if(sret != 0){
            CLibrary.INSTANCE.printf("CAN_CloseBoard error, ret=%d\n", sret);
            return;
        }
    }
}
